% Updated 06/26/13 for 3mb lib version 13.xx.xxx
% Output version 4.xx.xxx
% Demo application for reading in 3mb animat state data.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input:
%   szFileName: A string that is the name of the .3mb binary file to read
%               from.
%   Animats:    An array of specified animats to display.  If empty, this
%               script displays animat from
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function animatState = Plot3MbBinaryData(...
    szFileName,... 3mb file name.to open and read from
    SinglePlot,... normally false.  Displays single plot at a time.
    Animats,...    an optional argument that is an array of specific ani
    IterationRange ...an optional argument that is an array of specific its
    )

% Initialize the return value.
animatState = [];

if nargin < 2
    return;
elseif nargin == 2
    Animats = [];
    IterationRange = [];
elseif nargin == 3
    IterationRange = [];
end
    


[fidBin, errorMsg] = fopen(szFileName, 'r');
if fidBin == -1
    disp(['Unable to open file ', szFileName, ': ', errorMsg]);
    return;
end

% Get the name of the file and path to save figures to.
figFileName = szFileName;
spaceLoc = max(strfind(figFileName, '.3mb'));
saveDirectory = sprintf('%s', figFileName(1:spaceLoc-1));

% Get just the file title to add to the figures title.
spaceLoc = max(strfind(szFileName, '.3mb'));
szFileName = szFileName(1:spaceLoc-1);
spaceLoc = max(strfind(szFileName, '\'));
szFileName = szFileName(spaceLoc+1:length(szFileName));

%-------------------------------------------------------------------------%
% Read the Scenario Parameters
%-------------------------------------------------------------------------%
sceParams = ReadScenarioParams(fidBin);
if isempty(sceParams)
    display('Read Scenario Params Error');
    return;
end


disp(sprintf(...
    'Plot3MbBinaryData Demo for 3MB binary output format version %d.%03d',...
    sceParams.binaryOutputVerSuper, sceParams.binaryOutputVerSub));
disp('Output generated using:');
disp(sprintf(...
    '   3MB library version %d.%02d',...
    sceParams.libVersionSuper, sceParams.libVersionSub));
disp(sprintf(...
    '   Species (Builder) version %d.%02d',...
    sceParams.speciesBuilderVerSuper, sceParams.speciesBuilderVerSub));


%-------------------------------------------------------------------------%
% Post Scenario-Run Analysis
%---------------------------%
% If the scenario was configured to perform post simulation analysis, read
% it in.  ReadStatAnalysis() doesn't actually need to be called.  It is
% placed here to demo how to access the information contained in the post
% analysis file region.

% commented out because this version of 3MB no longer uses this post-run
% analysis in the output file, although the regions persist in the output
% file.
%  if sceParams.binOutConfig.postRunAnalysis == 0
%      statAnalysis = ReadStatAnalysis(fidBin, sceParams);
%  end

%-------------------------------------------------------------------------%
% Read the Species Description and the animat association to those
% species.
%---------%
% Neither ReadAnimatDescription() nor
% AddAnimatDescriptionToSpeDescription() need to be called.  They are shown
% here to demo how to access the information contained in their associated
% file regions.
% if sceParams.binOutConfig.speInfAndAnimatAsscn == 1
%     speDesc = ReadSpeciesDescriptions(fidBin, sceParams);
%     speDesc = ...
%         AddAnimatDescriptionToSpeDescription(fidBin, sceParams, speDesc);
% end

%---------------------------------------------------------------------%
% Conserve Memory!
%
% The animat's state is composed of several values.  Because this
% script reads into and holds every animat's state for the entire
% duration of the run memory demands may be intense.  The following
% struct is a means to save memory by specifying which state should be
% read in from file and stored.  Set the value of the members of this
% struct to 1 to hold the values and 0 to not.
%---------------------------------------------------------------------%
stateSelect = GetAnimatStateVarsStuct('');
stateSelect.id = 1; %animat unique ID
stateSelect.clock = 1; % current iteration
stateSelect.coords = 1; %coordinates
stateSelect.depth = 1; %depth
stateSelect.bearing = 1; %bearing
stateSelect.diveRate = 1; %dive rate
stateSelect.travelRate = 1; %travel rate
stateSelect.cExposure = 0; %cummulative acoustic exposure
stateSelect.iExposure = 0; %instantaneous acoustic exposure
stateSelect.relAngle = 0; %acoustic source position angle
stateSelect.secAvert = 0; %seconds in aversion
stateSelect.bathyDepth = 1; %bathy depth at current location
stateSelect.salinity = 0; %salinity at current location
stateSelect.temperature = 0; %temperature current location
stateSelect.behavior = 1; %current behavior
stateSelect.thrshFlag = 0; % threshold flag
stateSelect.diveActivity = 1;
stateSelect.overrideBehavior = 0;
stateSelect.targetDepth = 1;
stateSelect.calculatedDepth = 1;
stateSelect.xyDistance = 0;
stateSelect.riskValue = 0;
stateSelect.subID = 0;
stateSelect.offscreenReset = 1;
stateSelect.animatSel = Animats;
stateSelect.iterationRange = IterationRange;

numAnimatsToRead = sceParams.totalNumAnimats;


animatState = ReadAnimatPopulationState(fidBin, sceParams, stateSelect, ...
    numAnimatsToRead);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Demo reading in 3MB seeding polygons
% Added for 
% Set the file pointer to the location where 3MB polygons are stored.
if 0 ~= sceParams.regionSize.mmmbPolygons
    fseek(fidBin, sceParams.filePointers.mmmbPolygons, 'bof');
    mmmbPolygons = Read3MBPolygons(fidBin); %#ok<NASGU>
end

if 0 ~= sceParams.regionSize.smbPolygons
    fseek(fidBin, sceParams.filePointers.smbPolygons, 'bof');
    smbPolygons = ReadSmbPolygons(fidBin); %#ok<NASGU>
end

% Verification 
% smbPolygons.polygonClusterCount
% smbPolygons.clusterArray(1).polygonArray(1).vertices(1)
% smbPolygons.innerOpsBox.vertices(1);
% smbPolygons.outerOpsBox.vertices(1);
%end verification.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear smbPolygons;
% All state data is now in memory, so close the file.
if 0 ~= fclose(fidBin)
    display('warning: File handle did not close');
end

%-------------------------------------------------------------------------%
% Plot the data and save the figures.
%-------------------------------------------------------------------------%
PLOT = struct('data', []);
plotData1 = PLOT;
plotData2 = [];

plotColor = ['b', 'r', 'g', 'k', 'c', 'm'];
plotColorIndex = 1;

numAnimats = length(animatState);

startTime = sceParams.startTime;

for k=1:numAnimats
    display(sprintf('Plotting Animat: %05d', k));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Testing and development or to just limit animat displays to the last one.    
    % Skip all but the last
%      if k ~= numAnimats
%          continue;
%      end
    

    % Pause very 500 animats
%     if mod(k, 500) == 0
%         pause(20);
%     end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   
    figure(k);

    if stateSelect.clock == 1
        startTime = animatState(k).timeOfDay(1);
    end

    
    %---------------------------------------------------------------------%
    % Animat coordinates
    %-------------------%
    if sceParams.binOutConfig.animat.coordinate == 1 && ...
            stateSelect.coords == 1
        hdlSubplot = subplot(3, 2, 1);
        PlotAnimatCoordinates2(szFileName, animatState, plotColor);
        set(hdlSubplot, 'FontSize', 6);
    end
    
    %-----------------------------------------------------------------%
    % TravelRate
    %------------%
    if sceParams.binOutConfig.animat.travelRate == 1 && ...
            stateSelect.travelRate == 1
        hdlSubplot = subplot(3, 2, 2);
        plotData1.data = animatState(k).travelRate;
        PlotTimeData2(plotColor(plotColorIndex), k, animatState,...
            sceParams, plotData1, szFileName,  ...
            sprintf('Speed Profile Animat %d', k), 'Travel Rate (m/s)',...
            [], [], [], startTime);
        set(hdlSubplot, 'FontSize', 6);
    end

    %-----------------------------------------------------------------%
    % Bearing
    %--------%
    if sceParams.binOutConfig.animat.bearing == 1 && ...
            stateSelect.bearing == 1
        hdlSubplot = subplot(3, 2, 3);
        plotData1.data = animatState(k).bearing;
        PlotTimeData2(plotColor(plotColorIndex), k, animatState,...
            sceParams, plotData1, szFileName,...
            sprintf('Bearing Profile Animat %d', k), 'Heading (deg)',...
            [], [], [], startTime);
        set(hdlSubplot, 'FontSize', 6);
    end
    
    %---------------------------------------------------------------------%
    % Depth
    %-------%
    if sceParams.binOutConfig.animat.depth == 1 && ...
            stateSelect.depth == 1
        hdlSubplot = subplot(3, 2, 4);
        plotData1.data = animatState(k).depth;

        if sceParams.binOutConfig.animat.bathyDepth == 1 && ...
                stateSelect.bathyDepth == 1
            plotData2.data = animatState(k).bathyDepth;
        else
            plotData2.data = zeros(1, length(animatState(k).depth));
        end
        if sceParams.binOutConfig.animat.targetDepth == 1 && ...
                stateSelect.targetDepth == 1
            plotData3.data = animatState(k).targetDepth;
        else
            plotData3.data = zeros(1, length(animatState(k).depth));
        end
        if sceParams.binOutConfig.animat.calcDepth == 1 && ...
                stateSelect.calculatedDepth == 1
            plotData4.data = animatState(k).calculatedDepth;
        else
            plotData4.data = zeros(1, length(animatState(k).depth));
        end

        % Get the minimum depth (greatest depth) the animat reaches.
        minDepth = min(plotData1.data);
        %    excessiveDepthIndices = find(plotData2.data < minDepth - 100);
        %     if length(excessiveDepthIndices) ~= 0
        %         plotData2.data(excessiveDepthIndices) = minDepth - 100;
        %     end
        PlotTimeData2(plotColor(plotColorIndex), k, animatState, ...
            sceParams, plotData1, szFileName,...
            sprintf('Dive Profile Animat %d', k), 'Depth (m)',...
            plotData2, plotData3, plotData4, startTime);
        set(hdlSubplot, 'FontSize', 6);
        axisLims = axis;
        if axisLims(3) < minDepth - 100
            axisLims(3) = minDepth - 100;
        end
        axisLims(4) = 10;
        axis(axisLims);
    end

    
    %-------------------------------------------------------------------------%
    % Dive Rate
    %----------%
    if sceParams.binOutConfig.animat.diveRate == 1 && ...
            stateSelect.diveRate == 1
        hdlSubplot = subplot(3, 2, 5);
        plotData1.data = animatState(k).diveRate;

        for m=1:length(plotData1.data)
            if (animatState(k).diveActivity(m) == 3 || ...
                    animatState(k).diveActivity(m) == 5)
                plotData1.data(m) = -plotData1.data(m);
            end
        end

        PlotTimeData2(plotColor(plotColorIndex), k, animatState, ...
            sceParams, plotData1, szFileName, ...
            sprintf('Dive Rate Profile Animat %d', k) , ...
            'Dive Rate (m/s)', [], [], [], startTime);
        set(hdlSubplot, 'FontSize', 6);
    end

    %-------------------------------------------------------------------------%
    % Behavior
    %----------%
    if sceParams.binOutConfig.animat.packedData == 1 && ...
            stateSelect.behavior == 1
        hdlSubplot = subplot(3, 2, 6);
        plotData1.data = animatState(k).behavior+1;
        PlotTimeData2(plotColor(plotColorIndex), k, animatState, ...
            sceParams, plotData1, szFileName, ...
            sprintf('Behavior Profile Animat %d', k),...
            'Behavior State', [], [], [], startTime);

        % Maintain the axis created before the call to set automatically
        % changes them.
        axisLims = axis;
        set(hdlSubplot, 'FontSize', 6);
        axisLims(3) = -0.5;
        if min(plotData1.data) == 0 && max(plotData1.data) == 0
            axisLims(4) = 1;
        else
            axisLims(4) = 0.5 + max(plotData1.data);
        end
        axis(axisLims);
    end

    %-------------------------------------------------------------------------%
    plotColorIndex = plotColorIndex + 1;
    if plotColorIndex > length(plotColor)
        plotColorIndex = 1;
    end
   
    outFileName=sprintf('%s\\animat%02d', saveDirectory, k);    
    % Save options
    %saveas(gcf, outFileName, 'fig');
    %saveas(gcf, outFileName, 'jpg');

    if length(dir(saveDirectory)) == 0
        mkdir(saveDirectory);
    end
    
    saveas(gcf, outFileName, 'bmp');
    %--------------------%
    
    if SinglePlot == 1 && k-1 > 0
        close(k-1)
    end    
end