% Returns an SPECIESDESC array of species descriptions.  Each array member
% represents a species in the scenario and holds the species title, the
% number of behaviors defined for that species, an array of the behavior
% names of that species, the shore following depth of that species, and the
% number of animats that species was populated with.
function SPECIESDESC = ReadSpeciesDescriptions(fid, SceParams)

% Updated for 3mb lib version 4.00 on 09/02/08

fseek(fid, SceParams.filePointers.speDescription, 'bof');


% Constants
SPE_TITLE_BYTES = 256; % bytes
SPE_RESEVE_SPACE = 16; % bytes
BEH_NAME_BYTES = 64; % bytes

DATETIME = struct(...
    'year', 0, ... (UINT32)
    'month', 0, ... (UINT32)
    'day', 0, ... (UINT32)
    'hour', 0, ... (UINT32)
    'min', 0, ... (UINT32)
    'sec', 0 ... (UINT32)
    );

%--------------------------------------------------------%
% Read the Species Description
%-----------------------------%
% This Matlab struct corresponds to the SPECIESBINOUTINF struct in the C++
% 3mb library code.  The SPECIESBINOUTINF itself is made up of other
% structs.
SPECIESDESC = struct(...
    'szTitle', 0, ...
    ... 'szHeader', 0, ... A header that identifies the begining of the SPECIESSPECIFICATION struct
    'mmmbVersionSuper', 0, ... (UINT32)
    'mmmbVersionSub', 0, ... (UINT32)
    'speVersionSuper', 0, ... (UINT32)
    'speVersionSub', 0, ... (UINT32)
    'dateTime', DATETIME, ...
    'speciesID', 0, ... (UINT32)
    'numBehaviors', 0, ... (UINT32),
    'speciesGroup', 0, ... (UINT32)enumeration: MYSTICETE, HFODONTOCETE, MFODONTOCETE,
                       ...                    SPECIALCONSIDRTNS, PHOCID, OTARIID,
                       ...                    OTHERMARINEMAMMALS, SEATURTLES
    ... 'speciesName', 0, ... (uint)enumeration referencing the name of the species.   
    'deepWaterSeedLimitEnabled', 0, ...
    'shoreFollowDepth', 0, ... (double)
    'minSeedingDepth', 0, ... (double)
    'deepWaterSeedingDepthLimit', 0, ... (double)
    'szShortDescription', 0, ... (96 bytes)
    'szLongDescription', 0, ...  (4096 bytes)
    'behaviorNameArray', 0, ...
    'numAnimats', -1); % The number of animats in this species.  NOTE: the number of
                      % animats in a species IS NOT actually with the
                      % species description in the binary file.  This field
                      % is filled in when a call to the function
                      % ReadAnimatDescription.m is made. Not part of any
                      % C++ code struct.

for n=1:SceParams.numSpecies,
    %The first SPE_TITLE_BYTES bytes are a buffer for the species title (no extensions)
    buffer = fread(fid, SPE_TITLE_BYTES, 'char');
    SPECIESDESC(n).szTitle = sprintf('%s', char(buffer)); 

    % Skip past string header (szHeader).
    fseek(fid, 16, 'cof'); %4 bytes
        
    % Read in the 3mb version numbers.
    SPECIESDESC(n).mmmbVersionSuper = fread(fid, 1, 'uint32');
    SPECIESDESC(n).mmmbVersionSub = fread(fid, 1, 'uint32');

    % Read saved date/time/unique id information (szHeader).
    SPECIESDESC(n).speVersionSuper = fread(fid, 1, 'uint32');
    SPECIESDESC(n).speVersionSub = fread(fid, 1, 'uint32');
    
    SPECIESDESC(n).dateTime.year = fread(fid, 1, 'uint32');
    SPECIESDESC(n).dateTime.month = fread(fid, 1, 'uint32');
    SPECIESDESC(n).dateTime.day = fread(fid, 1, 'uint32');
    SPECIESDESC(n).dateTime.hour = fread(fid, 1, 'uint32');
    SPECIESDESC(n).dateTime.min = fread(fid, 1, 'uint32');
    SPECIESDESC(n).dateTime.sec = fread(fid, 1, 'uint32');
    SPECIESDESC(n).speciesID = fread(fid, 1, 'uint32');

    % Read in the number of behaviors this species engages in.
    SPECIESDESC(n).numBehaviors = fread(fid, 1, 'uint32');   % 4 bytes
      
    % Read in the species group this species belongs to..
    SPECIESDESC(n).speciesGroup = fread(fid, 1, 'uint32');   % 4 bytes
    
    % Skip past speciesName.
    fseek(fid, 4, 'cof'); %4 bytes

    
    SPECIESDESC(n).deepWaterSeedLimitEnabled = fread(fid, 1, 'uint32');

    % Skip past reserved space _reservedSpace1.
    fseek(fid, 36, 'cof');

    % Read in the shore following depth for this species.
    SPECIESDESC(n).shoreFollowDepth = fread(fid, 1, 'double');   % 8 bytes
    
    % Read in the minimum seeding depth of the bathymetry for this species.
    SPECIESDESC(n).minSeedingDepth = fread(fid, 1, 'double');   % 8 bytes

    % Read in the minimum seeding depth of the bathymetry for this species.
    SPECIESDESC(n).deepWaterSeedingDepthLimit = fread(fid, 1, 'double');   % 8 bytes
    
    % Skip past reserved space _reservedSpace2.
    fseek(fid, 56, 'cof');
    
    % Read in the short description
    buffer = fread(fid, 96, 'char');
    SPECIESDESC(n).szShortDescription = sprintf('%s', char(buffer)); 
    
    % Skip past in the long description
    fseek(fid, 4096, 'cof');
    %[buffer, count] = fread(fid, 4096, 'char');
    %[SPECIESDESC(n).szLongDescription, errorMsg] = sprintf('%s', buffer); 
   
    SPECIESDESC(n).numAnimats = 0; % This will be filled in when call
                                   %to ReadAnimatDescription.m is made.
    % Read in each behavior's name
    if SPECIESDESC(n).numBehaviors > 0
        %zero out memory.
        SPECIESDESC(n).behaviorNameArray = zeros(SPECIESDESC.numBehaviors, 64);
    end
    
    % For now skip over the behavior names.Additional work required if
    % behavior names are ever actually used.
    for k=1:SPECIESDESC(n).numBehaviors,
        fseek(fid, 64, 'cof'); %4 bytes
        %SPECIESDESC(n).behaviorNameArray(k,:) = fread(fid, BEH_NAME_BYTES, 'char');
        
        % Can't do this.
        %[buffer, count] = fread(fid, BEH_NAME_BYTES, 'char');   % BEH_NAME_BYTES bytes
        %SPECIESDESC(n).behaviorNameArray(k) = sprintf('%s', buffer);
    end
    % Skip past reserved space _RESERVED.
    fseek(fid, 16, 'cof');   
end